<?php
/**
 * A Socket Container class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2014 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class SocketContainer extends BaseContainer implements Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this Container class and prepares it for usage
	 *
	 * @param	$socketResource		A valid socket resource
	 * @param	$protocolInstance	A HandleableProtocol instance
	 * @param	$packageData		Raw package data
	 * @return	$containerInstance	An instance of this Container class
	 */
	public static final function createSocketContainer ($socketResource, HandleableProtocol $protocolInstance = NULL, array $packageData = array()) {
		// Get a new instance
		$containerInstance = new SocketContainer();

		// Remove unneeded entries
		unset($packageData[NetworkPackage::PACKAGE_DATA_CONTENT]);
		unset($packageData[NetworkPackage::PACKAGE_DATA_SIGNATURE]);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']:socketResource=' . $socketResource . ',packageData='.print_r($packageData, TRUE));

		// Set the resource ...
		$containerInstance->setSocketResource($socketResource);

		// ..., protocol instance ...
		$containerInstance->setProtocolInstance($protocolInstance);

		// ... and package data
		$containerInstance->setPackageData($packageData);

		// Return the prepared instance
		return $containerInstance;
	}

	/**
	 * Checks whether the given Universal Node Locator matches with the one from package data
	 *
	 * @param	$unlInstance	An instance of a LocateableNode class
	 * @return	$matches		Whether $address matches with the one from package data
	 */
	public final function ifAddressMatches (LocateableNode $unlInstance) {
		// Get current package data
		$packageData = $this->getPackageData();

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER: unlInstance=' . print_r($unlInstance, TRUE) . ',packageData=' . print_r($packageData, TRUE));

		// So, does both match?
		die(__METHOD__ . ': Unfinished.' . PHP_EOL);
		$matches = ((isset($packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT])) && ($packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT] === $unlInstance));

		// Return result
		return $matches;
	}

	/**
	 * Checks whether the given socket matches with stored
	 *
	 * @param	$unlInstance	An instance of a LocateableNode class
	 * @return	$matches		Whether given socket matches
	 */
	public final function ifSocketResourceMatches ($socketResource) {
		// Debug message
		/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER: socketResource[' . gettype($socketResource) . ']=' .$socketResource . ',storedResource[' . gettype($this->getSocketResource()) . ']=' . $this->getSocketResource());

		// So, does both match?
		$matches = ((is_resource($socketResource)) && ($socketResource === $this->getSocketResource()));

		// Return result
		return $matches;
	}
}

// [EOF]
?>
